/*

das Puzzle Brett

+---------+---------+---------+
|         |         |         |
|         |         |         |
|    1   c|a   2   c|a   3    |
|         |         |         |
|    d    |    d    |    d    |
+---------+---------+---------+
|    b    |    b    |    b    |
|         |         |         |
|    4   c|a   5   c|a   6   c|
|         |         |         |
|    d    |    b    |    d    |
+---------+---------+---------+
|    b    |    d    |    b    |
|         |         |         |
|    7   c|a   8   c|a   9    |
|         |         |         |
|         |         |         |
+---------+---------+---------+


*/

:- use_module(library(fd)).

karten(Karten) :-
	Karten = [
	  karte(0, pinguin + r, katze   + r, schwein + l, maus  + l),
          karte(1, pinguin + r, katze   + l, schwein + l, maus  + r),
	  karte(2, pinguin + r, katze   + l, schwein + r, maus  + l),
	  karte(3, pinguin + r, schwein + l, maus    + l, katze + r),
	  karte(4, pinguin + l, katze   + r, schwein + l, maus  + r),
	  karte(5, pinguin + l, schwein + r, maus    + l, katze + r),
	  karte(6, pinguin + l, schwein + r, katze   + l, maus  + r),
	  karte(7, pinguin + l, maus    + r, schwein + l, katze + r),
	  karte(8, pinguin + l, schwein + r, maus    + r, katze + l)].

brett([F1,F2,F3,F4,F5,F6,F7,F8,F9]) :-
						% die Brett-Topologie:
						% die Belegung der Rnder ist beleibeig
	F1 = feld(_, _        , _        , T1c + S1c, T1d + S1d),
	F2 = feld(_, T2a + S2a, _        , T2c + S2c, T2d + S2d),
	F3 = feld(_, T3a + S3a, _        , _        , T3d + S3d),
	F4 = feld(_, _        , T4b + S4b, T4c + S4c, T4d + S4d),
	F5 = feld(_, T5a + S5a, T5b + S5b, T5c + S5c, T5d + S5d),
	F6 = feld(_, T6a + S6a, T6b + S6b, _        , T6d + S6d),
	F7 = feld(_, _        , T7b + S7b, T7c + S7c, _        ),
	F8 = feld(_, T8a + S8a, T8b + S8b, T8c + S8c, _        ),
	F9 = feld(_, T9a + S9a, T9b + S9b, _        , _        ),

						% die Bilder an den Grenzen mssen bereinstimmen
	T1c = T2a, T2c = T3a,
	T4c = T5a, T5c = T6a,
	T7c = T8a, T8c = T9a,
	T1d = T4b, T4d = T7b,
	T2d = T5b, T5d = T8b,
	T3d = T6b, T6d = T9b,

						% die Werte (links - rechts) mssen verschieden sein
						% \= geht nicht als Operation, da die Variablen
						% noch frei sind
	S1c #\= S2a, S2c #\= S3a,
	S4c #\= S5a, S5c #\= S6a,
	S7c #\= S8a, S8c #\= S9a,
	S1d #\= S4b, S4d #\= S7b,
	S2d #\= S5b, S5d #\= S8b,
	S3d #\= S6b, S6d #\= S9b.


% Karten drehen

drehen(karte(I,S1,S2,S3,S4),feld(J,S1,S2,S3,S4)) :- J is 4 * I + 0.
drehen(karte(I,S1,S2,S3,S4),feld(J,S2,S3,S4,S1)) :- J is 4 * I + 1.
drehen(karte(I,S1,S2,S3,S4),feld(J,S3,S4,S1,S2)) :- J is 4 * I + 2.
drehen(karte(I,S1,S2,S3,S4),feld(J,S4,S1,S2,S3)) :- J is 4 * I + 3.

% Felder belegen

belegen([],[],[]).

belegen([Feld | RestFelder],Karten,[Nr | RestNrn]) :-
	delete(Karte,Karten,RestKarten),
	drehen(Karte,Feld),
	versuchen(Feld,Nr),
	belegen(RestFelder,RestKarten,RestNrn).

versuchen(feld(Feld,_,_,_,_),Nr) :-
	write('setField '),
	write(Nr),
	write(' '),
	write(Feld),
	nl.

versuchen(feld(_,_,_,_,_),Nr) :-
	write('setField '),
	write(Nr),
	write(' '),
	write(-1),
	nl,
	fail.

loesen(Brett) :-
	karten(Karten),
	brett(Brett),
	Brett  = [F0,F1,F2,F3,F4,F5,F6,F7,F8],
	Brett1 = [F4,F1,F5,F2,F7,F8,F3,F6,F0],
	Reihenfolge = [4,1,5,2,7,8,3,6,0],
	belegen(Brett1,Karten,Reihenfolge).

ausgeben(Brett,I,Zeit) :-
	write('Lsung gefunden in '),write(Zeit),write(' Sek.'),nl,
	ausgeben(Brett,I).

ausgeben([],_).
ausgeben([Feld|Rest],I) :-
	I1 is I + 1,
	write('Feld '),write(I1),write(':'),nl,
	ausgeben(Feld),
	ausgeben(Rest,I1).

ausgeben(feld(I,L+LR,O+OR,R+RR,U+UR)) :-
	write('    links : '),write(L),ausgeben(LR),nl,
	write('    oben  : '),write(O),ausgeben(OR),nl,
	write('    rechts: '),write(R),ausgeben(RR),nl,
	write('    unten : '),write(U),ausgeben(UR),nl,
	write('    Bild  : '),write(I),nl.

ausgeben(r) :-
	write('	richtig herum').

ausgeben(l) :-
	write('	auf dem Kopf').

loesen :-
	cputime(Start),
	loesen(Brett),
	cputime(Stop),
	Time is Stop - Start,
	ausgeben(Brett,0,Time).
