# $Id: vector.tcl,v 1.1 1996/11/17 14:53:52 uwe Exp $
#
# vector arithmetic

proc scaleVec {x y sc} {
    return [list [expr $x * $sc] [expr $y * $sc]]
}

proc addVec {x0 y0 x1 y1} {
    return [list [expr $x0 + $x1] [expr $y0 + $y1]]
}

proc subVec {x0 y0 x1 y1} {
    return [list [expr $x0 - $x1] [expr $y0 - $y1]]
}

proc normVec {x y} {
    set len [lenVec $x $y]
    if {$len == 0} {error "can't normalize null vector"}
    return [list [expr $x / double($len)] [expr $y / double($len)]]
}

proc iprodVec {x0 y0 x1 y1} {
    return [expr $x0 * $x1 + $y0 * $y1]
}

proc len2Vec {x0 y0} {
    return [iprodVec $x0 $y0 $x0 $y0]
}

proc lenVec {x0 y0} {
    return [expr sqrt([len2Vec $x0 $y0])]
}

proc multVec {x1 x2 y1 y2 x y} {
    return [list \
		[expr $x1 * $x + $x2 * $y] \
		[expr $y1 * $x + $y2 * $y]]
}

# create a rotation matrix from a given vector
proc rotMat {x y} {
    set norm [normVec $x $y]
    set x0 [lindex $norm 0]
    set y0 [lindex $norm 1]
    return [list $x0 $y0 $y0 [expr 0 - $x0]]
}

# compute the angle of a vector and the x-axis
proc 2angleVec {x y} {
    return [eval 2angleNormVec [normVec $x $y]]
}

proc 2angleNormVec {x y} {
    set angle [expr acos($x) / 3.14159 * 180.0 ]
    if {$y < 0.0} {
	set angle [expr 360.0 - $angle]
    }
    return $angle
}

# rotate a vector 90 degrees
proc rotBy90 {x y} {
    return [list $y [expr - $x]]
}

proc rotBy90xN {x y n} {
    set p [list $x $y]
    for {set i [expr $n % 4]} {$i != 0} {incr i -1} {
	set p [eval rotBy90 $p]
    }
    return $p
}

